<?php
namespace Teknisa\MAN\Services;

use Zeedhi\Framework\DB\StoredProcedure\StoredProcedure;
use Zeedhi\Framework\DB\StoredProcedure\Param;
use Teknisa\MAN\Services\NovoCodigo;

class RetiradaProdutoAcabadoService {

    /* Identificador do tipo de operacao que o sistema esta sendo realizado => 'E' para 'Lançamento de Entrada' */
    const ENTRADA       = 'E';
    /* Identificador do tipo de operacao que o sistema esta sendo realizado => 'S' para 'Lançamento de Saida'   */
    const SAIDA         = 'S';
    /* Identificador do tipo de operacao que o sistema esta sendo realizado => 'A' para 'Ajuste de Inventario'
     * e 'Implantacao de Estoque'
     */
    const AJUSTE_IMP    = 'A';
    const TIPO_CUSTO_PADRAO = '03';
    const ESTOQUE_VAZIO = ' ';
    public $entityManager;
    protected $paramsDelete = array();
    
    public function __construct(\Doctrine\ORM\EntityManager $entityManager)  {
        $this->entityManager = $entityManager;
    }
	
	// Parâmetros: CDALMOXARIFE, CDFILIAL, NRLANCESTQ, NRSEQUITEM
	public function atualizarProduto ($params) {
	    try {
	        
            $this->entityManager->getConnection()->executeQuery(self::UPDATE_ALMOXARIFADO, $params);
	        
	    } catch (\Exception $e) {
            $this->entityManager->getConnection()->rollBack();
            throw $e;
        }
        return;
	}
	
	public function atualizarEstoque($params) {
	   try {
            $procedure = new StoredProcedure($this->entityManager->getConnection(), 'ATUALIZA_ESTOQUE');
            
            $procedure->addParam(new Param('P_FILIMOVI', Param::PARAM_INPUT, $params["P_FILIMOVI"], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_DTINICIAL', Param::PARAM_INPUT, $params["P_DTINICIAL"], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_DTFINAL', Param::PARAM_INPUT, $params["P_DTFINAL"], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_FILILANCTO', Param::PARAM_INPUT, $params["P_FILILANCTO"], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_NRLANCTO', Param::PARAM_INPUT, $params["P_NRLANCTO"], Param::PARAM_TYPE_STR));
            $procedure->addParam(new Param('P_IDBLOQTRANS', Param::PARAM_INPUT, null, Param::PARAM_TYPE_STR));
            
            $procedure->execute();  
        } catch (\Exception $e) {
            throw $e;
        }  
	}
	
	public function getNovoCodigo ($CDCONTADOR, $NRORG) {
	    return NovoCodigo::geraCodigo($this->entityManager->getConnection(), $CDCONTADOR, $NRORG, 10);
	}
	
	public function executarQuery ($query, $params) {
	    try {
            $this->entityManager->getConnection()->executeQuery($query, $params);
	    } catch (\Exception $e) {
	        throw new \Exception($e->getMessage());
            $this->entityManager->getConnection()->rollBack();
        }
	}
	
	public function executarQuerySelect ($query, $params) {
	    return $this->entityManager->getConnection()->fetchAll($query, $params);
	}
	
	public function consultaCustoQuantidadeProdutoEstoque(
        $filial, $tipoCusto, $produto, $almoxarifado, $localEstoque, $lote, $subLote, $dtLancamento
    )
    {
        $sql = <<<SQL
        SELECT NVL(CUSTO_PRODUTO_LCEST(:P_CDFILIAL, D.CDPRODUTO, :P_INTPCUSTO, :P_DTLANC, SYSDATE,
               D.CDALMOXARIFE, D.CDLOCALESTOQ, D.NRLOTEESTQ, D.NRSUBLOTE),0) * P.VRFATOCONV AS VRCUSTOPROD,
               (D.QTESTOQDIA/P.VRFATOCONV) AS QTESTOQDIA
          FROM PRODUTO P, POSESTDIA D, LOTE L
         WHERE P.CDPRODUTO = :P_CDPRODUTO
           AND D.CDFILIAL = :P_CDFILIAL
           AND D.CDALMOXARIFE = :P_CDALMOXARIFE
           AND D.CDLOCALESTOQ = :P_CDLOCALESTOQ
           AND D.NRLOTEESTQ = :P_NRLOTEESTQ
           AND D.NRSUBLOTE = :P_NRSUBLOTE
           AND D.CDPRODUTO = P.CDPRODESTO
           AND (D.QTESTOQDIA > 0)
           AND D.CDFILIAL = L.CDFILIAL
           AND D.NRLOTEESTQ = L.NRLOTEESTQ
           AND D.NRSUBLOTE = L.NRSUBLOTE
           AND D.CDPRODUTO = L.CDPRODUTO
           AND D.DTPOSIESTQ = (SELECT MAX(B.DTPOSIESTQ)
                                 FROM POSESTDIA B
                                WHERE B.CDFILIAL     = D.CDFILIAL
                                  AND B.CDPRODUTO    = D.CDPRODUTO
                                  AND B.CDALMOXARIFE = D.CDALMOXARIFE
                                  AND B.CDLOCALESTOQ = D.CDLOCALESTOQ
                                  AND B.NRLOTEESTQ   = D.NRLOTEESTQ
                                  AND B.NRSUBLOTE    = D.NRSUBLOTE
                                  AND B.DTPOSIESTQ   <= TO_DATE(:P_DTLANC) + 1)
SQL;

        try{
            $res = $this->entityManager->getConnection()->fetchAssoc($sql, $params);
            return $res;
        } catch(\Exception $e) {
            throw $e;
        }
    }
    
    public function consultaCustoQuantidadeProduto(
        $filial, $produto, $dtLancamento, $tipoCusto = self::TIPO_CUSTO_PADRAO,
        $almoxarifado = self::ESTOQUE_VAZIO, $localEstoque = self::ESTOQUE_VAZIO,
        $lote = self::ESTOQUE_VAZIO, $subLote = self::ESTOQUE_VAZIO
    ){

        $params = array(
            "P_CDFILIAL" => $filial,
            "P_INTPCUSTO" => $tipoCusto,
            "P_CDALMOXARIFE" => $almoxarifado,
            "P_CDLOCALESTOQ" => $localEstoque,
            "P_CDPRODUTO" => $produto,
            "P_NRLOTEESTQ" => $lote,
            "P_NRSUBLOTE" => $subLote,
            "P_DTLANC" => $dtLancamento);

        /*$custoQuantidade = $this->consultaCustoQuantidadeProdutoEstoque(
            $filial, $tipoCusto, $produto, $almoxarifado, $localEstoque, $lote, $subLote, $dtLancamento
        );*/
        
        $custoQuantidade = $this->consultaCusto($params);

        if (!$custoQuantidade)
            $custoQuantidade = array('QTESTOQDIA' => '0', 'VRCUSTOPROD' => '0');

        return $custoQuantidade;
    }
    
    private function consultaCusto ($params) {
        try {
            $resposta = $this->executarQuerySelect(self::QUERY_VALOR_PROD_SEM_ALMOXARIFADO, $params);
            return $resposta;
	    } catch (\Exception $e) {
	        throw new \Exception($e->getMessage());
            $this->entityManager->getConnection()->rollBack();
        }
    }
	
	const UPDATE_ALMOXARIFADO = "
	    UPDATE ITLANCTOEST
        SET
          CDALMOXARIFE = :CDALMOXARIFE
        WHERE
          CDFILIAL   = :CDFILIAL   AND
          NRLANCESTQ = :NRLANCESTQ AND
          NRSEQUITEM = :NRSEQUITEM";
          
    const QUERY_VALOR_PROD_SEM_ALMOXARIFADO = "
        SELECT NVL(CUSTO_PRODUTO_LCEST(:P_CDFILIAL, D.CDPRODUTO, :P_INTPCUSTO, :P_DTLANC, SYSDATE,
               D.CDALMOXARIFE, D.CDLOCALESTOQ, D.NRLOTEESTQ, D.NRSUBLOTE),0) * P.VRFATOCONV AS VRCUSTOPROD,
               (D.QTESTOQDIA/P.VRFATOCONV) AS QTESTOQDIA, D.CDALMOXARIFE
          FROM PRODUTO P, POSESTDIA D, LOTE L
         WHERE P.CDPRODUTO = :P_CDPRODUTO
           AND D.CDFILIAL = :P_CDFILIAL
           AND D.CDLOCALESTOQ = :P_CDLOCALESTOQ
           
           AND D.NRLOTEESTQ = :P_NRLOTEESTQ
           AND D.NRSUBLOTE = :P_NRSUBLOTE
           AND D.CDPRODUTO = P.CDPRODESTO
           AND (D.QTESTOQDIA > 0)
           AND D.CDFILIAL = L.CDFILIAL
           AND D.NRLOTEESTQ = L.NRLOTEESTQ
           AND D.NRSUBLOTE = L.NRSUBLOTE
           AND D.CDPRODUTO = L.CDPRODUTO
           AND D.DTPOSIESTQ = (SELECT MAX(B.DTPOSIESTQ)
                                 FROM POSESTDIA B
                                WHERE B.CDFILIAL     = D.CDFILIAL
                                  AND B.CDPRODUTO    = D.CDPRODUTO
                                  AND B.CDALMOXARIFE = D.CDALMOXARIFE
                                  AND B.CDLOCALESTOQ = D.CDLOCALESTOQ
                                  AND B.NRLOTEESTQ   = D.NRLOTEESTQ
                                  AND B.NRSUBLOTE    = D.NRSUBLOTE
                                  AND B.DTPOSIESTQ   <= TO_DATE(:P_DTLANC) + 1)";
           
           const RESTO2 = "AND D.CDALMOXARIFE = :P_CDALMOXARIFE";
           
} 